<?php
namespace Rethumbify;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Media_Library {
    /**
     * Get image IDs based on the current settings and rules.
     *
     * @return array An array of image attachment IDs.
     */
    public function get_image_ids() {
        global $wpdb;
        $cache_key = 'rethumbify_image_ids';
        $image_ids = wp_cache_get($cache_key);

        if (false === $image_ids) {
            $query = $this->build_image_query();
            $image_ids = $wpdb->get_col($query);
            wp_cache_set($cache_key, $image_ids, '', 3600); // Cache for 1 hour
        }

        /**
         * Filter the array of image IDs to be processed.
         *
         * @since 1.0.0
         *
         * @param array $image_ids Array of attachment IDs to be processed.
         */
        return apply_filters('rethumbify_image_ids', $image_ids);
    }

    /**
     * Build the SQL query to fetch image IDs based on current settings.
     *
     * @return string The SQL query string.
     */
    protected function build_image_query() {
        global $wpdb;
        $options = get_option('rethumbify_settings', array());
        $regenerate_featured = isset($options['regenerate_featured']) ? $options['regenerate_featured'] : false;
        $selective_rules = isset($options['selective_rules']) ? $options['selective_rules'] : array();

        if ($regenerate_featured) {
            $query = "SELECT DISTINCT pm.meta_value AS ID 
                      FROM $wpdb->postmeta pm 
                      JOIN $wpdb->posts p ON p.ID = pm.meta_value 
                      WHERE pm.meta_key = '_thumbnail_id' 
                      AND p.post_type = 'attachment' 
                      AND p.post_mime_type LIKE 'image/%'";
        } else {
            $query = "SELECT ID FROM $wpdb->posts WHERE post_type = 'attachment' AND post_mime_type LIKE 'image/%'";
        }
        $where_clauses = array();

        //PRO:START
        if (!empty($selective_rules)) {
            foreach ($selective_rules as $rule) {
                if (isset($rule['condition'])) {
                    switch ($rule['condition']) {
                        case 'upload_date':
                            $date_query = $this->get_date_query($rule['upload_date']);
                            if ($date_query) {
                                $where_clauses[] = $date_query;
                            }
                            break;
                        case 'post_type':
                            if (isset($rule['post_type'])) {
                                if ($rule['post_type'] === 'product') {
                                    $where_clauses[] = $wpdb->prepare("ID IN (SELECT meta_value FROM $wpdb->postmeta WHERE meta_key = '_thumbnail_id' AND post_id IN (SELECT ID FROM $wpdb->posts WHERE post_type IN ('product', 'product_variation')))");
                                } else {
                                    $where_clauses[] = $wpdb->prepare("ID IN (SELECT meta_value FROM $wpdb->postmeta WHERE meta_key = '_thumbnail_id' AND post_id IN (SELECT ID FROM $wpdb->posts WHERE post_type = %s))", $rule['post_type']);
                                }
                            }
                            break;
                    }
                }
            }
        }
        //PRO:END

        if (!empty($where_clauses)) {
            $query .= " AND (" . implode(" OR ", $where_clauses) . ")";
        }

        return $query;
    }

    /**
     * Get the date query clause based on the upload date option.
     *
     * @param string $upload_date The upload date option.
     * @return string|null The SQL date query clause or null if invalid option.
     */
    private function get_date_query($upload_date) {
        global $wpdb;
        $now = current_time('mysql');
        
        switch ($upload_date) {
            case 'past_hour':
                $date = date('Y-m-d H:i:s', strtotime('-1 hour', strtotime($now)));
                break;
            case 'past_day':
                $date = date('Y-m-d H:i:s', strtotime('-1 day', strtotime($now)));
                break;
            case 'past_week':
                $date = date('Y-m-d H:i:s', strtotime('-1 week', strtotime($now)));
                break;
            case 'past_month':
                $date = date('Y-m-d H:i:s', strtotime('-1 month', strtotime($now)));
                break;
            case 'past_year':
                $date = date('Y-m-d H:i:s', strtotime('-1 year', strtotime($now)));
                break;
            default:
                return null;
        }

        return $wpdb->prepare("post_date >= %s", $date);
    }

    /**
     * Get the total number of images based on the current query.
     *
     * @return int The total number of images.
     */
    public function get_total_images() {
        global $wpdb;
        $cache_key = 'rethumbify_total_images';
        $total = wp_cache_get($cache_key);

        if (false === $total) {
            $query = $this->build_image_query();
            $wpdb->query("SELECT SQL_CALC_FOUND_ROWS 1 FROM ($query) AS subquery LIMIT 1");
            $total = (int) $wpdb->get_var("SELECT FOUND_ROWS()");

            wp_cache_set($cache_key, $total, '', 3600); // Cache for 1 hour
        }

        return $total;
    }
}