<?php
namespace Rethumbify;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

use WP_Error;
use Exception;
class Regenerate_Thumbnails {

    /**
     * Get the full size path for a given attachment ID.
     *
     * @param int $attachment_id The ID of the attachment.
     * @return string|WP_Error The full size path or WP_Error if not found.
     */
    private function get_full_size_path( $attachment_id ) {
        Logger::debug( sprintf( 'Getting full size path for attachment ID: %d', $attachment_id ) );
        
        $file = get_attached_file( $attachment_id );
        $basename = wp_basename( $file );

        // Check for edited version first
        if ( preg_match( '/-e[0-9]{13}\./', $basename ) ) {
            Logger::debug( sprintf( 'Using edited version of image: %s', $basename ) );
        } elseif ( function_exists( 'wp_get_original_image_path' ) ) {
            // Try to get the original file
            $original_file = wp_get_original_image_path( $attachment_id );
            if ( $original_file && file_exists( $original_file ) ) {
                Logger::debug( sprintf( 'Using original image file: %s', $original_file ) );
                $file = $original_file;
            }
        }
        
        $file = apply_filters( 'rethumbify_full_size_path', $file, $attachment_id );
        Logger::debug( sprintf( 'Final image path: %s', $file ) );

        // Check if the file exists
        if ( ! $file || ! file_exists( $file ) ) {
            Logger::error( sprintf( 'File not found for attachment ID %d. Path attempted: %s', $attachment_id, $file ) );
            return new WP_Error( 'file_not_found', self::get_error_message( 'file_not_found' ) );
        }

        return $file;
    }

    /**
     * Regenerate thumbnails for a given attachment ID.
     *
     * @param int $attachment_id The ID of the attachment to regenerate thumbnails for.
     * @return bool|WP_Error True on success, WP_Error object on failure.
     */
    public function regenerate_thumbnails( $attachment_id ) {
        Logger::info( sprintf( 'Starting thumbnail regeneration for attachment ID: %d', $attachment_id ) );

        // Ensure necessary functions are available
        if ( ! function_exists( 'wp_generate_attachment_metadata' ) ) {
            Logger::debug( 'Loading image.php file for wp_generate_attachment_metadata function' );
            require_once( ABSPATH . 'wp-admin/includes/image.php' );
        }

        $file = $this->get_full_size_path( $attachment_id );
        if ( is_wp_error( $file ) ) {
            Logger::error( sprintf( 'Failed to get full size path: %s', $file->get_error_message() ) );
            return $file;
        }
       
        Logger::debug( sprintf( 'Generating attachment metadata for file: %s', $file ) );
        // Generate new metadata and thumbnails

        /* @var array|Object $metadata */
        $metadata = wp_generate_attachment_metadata( $attachment_id, $file );
        if ( is_wp_error( $metadata ) ) {
            Logger::error( sprintf( 'Metadata generation failed: %s', $metadata->get_error_message() ) );
            try {
                Logger::debug( 'Hooks for `wp_generate_attachment_metadata`: ' . wp_json_encode( $GLOBALS['wp_filter']['wp_generate_attachment_metadata'] ) );
            } catch ( Exception $e ) {
                // Do nothing
            }
            return $metadata;
        }

        if ( empty( $metadata ) ) {
            Logger::info( sprintf( 'Empty metadata generated for attachment ID: %d', $attachment_id ) );
            Logger::debug( sprintf( 'Generated metadata: %s', wp_json_encode( $metadata ) ) );
            return new WP_Error( 'metadata_generation_failed', self::get_error_message( 'metadata_generation_failed' ) );
        }

        // Update the attachment metadata
        Logger::debug( sprintf( 'Updating attachment metadata for ID: %d', $attachment_id ) );
        Logger::debug( sprintf( 'Generated metadata: %s', wp_json_encode( $metadata ) ) );
        wp_update_attachment_metadata( $attachment_id, $metadata );

        try{
            Logger::debug( 'Hooks for `wp_update_attachment_metadata`: ' . wp_json_encode( $GLOBALS['wp_filter']['wp_update_attachment_metadata'] ) );
        } catch ( Exception $e ) {
            // Do nothing
        }

        Logger::info( sprintf( 'Successfully regenerated thumbnails for attachment ID: %d', $attachment_id ) );
        return true;
    }

    /**
     * Get error message for a given error code.
     *
     * @param string $error_code The error code to match.
     * @return string The corresponding error message.
     */
    public static function get_error_message( $error_code ) {
        $default_messages = array(
            'file_not_found'             => __( 'The original image file was not found.', 'rethumbify' ),
            'metadata_generation_failed' => __( 'Failed to generate attachment metadata.', 'rethumbify' ),
            'default'                    => __( 'An unexpected error occurred.', 'rethumbify' ),
        );

        $error_messages = wp_parse_args( array(), $default_messages );
        return isset( $error_messages[ $error_code ] ) ? $error_messages[ $error_code ] : $error_messages['default'];
    }
}