<?php

namespace Rethumbify;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}
class Status {
    private $table_name;

    public function __construct() {
        global $wpdb;
        $this->table_name = $wpdb->prefix . 'rethumbify_status';
        $this->maybe_create_table();
    }

    private function maybe_create_table() {
        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE IF NOT EXISTS $this->table_name (
            id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
            attachment_id bigint(20) unsigned NOT NULL,
            status varchar(20) NOT NULL DEFAULT 'pending',
            error_code varchar(50) DEFAULT NULL,
            created_at datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            updated_at datetime NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            PRIMARY KEY  (id),
            KEY attachment_id (attachment_id),
            KEY status (status)
        ) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    }

    public function drop_table() {
        global $wpdb;
        $wpdb->query("DROP TABLE IF EXISTS $this->table_name");
    }

    public function get_failed_images(){
    global $wpdb;

    $failed_images = $wpdb->get_results(
        $wpdb->prepare(
            "SELECT attachment_id, error_code FROM {$this->table_name} WHERE status = %s",
            'failed'
        ),
        ARRAY_A
    );

    $result = array();
    if ( ! empty( $failed_images ) ) {
        foreach ( $failed_images as $image ) {
            $result[ $image['attachment_id'] ] = $image['error_code'];
        }
    }

    return $result;
    }

   
    public function is_completed(){
    global $wpdb;

    $total_count = $wpdb->get_var("SELECT COUNT(*) FROM {$this->table_name}");
    $completed_count = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$this->table_name} WHERE status = %s",
        'completed'
    ));

    $failed_count = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$this->table_name} WHERE status = %s",
        'failed'
    ));

    return ($total_count > 0 && $total_count == ($completed_count + $failed_count));
    }

    public function mark_as_pending($attachment_id) {
        global $wpdb;
    
        $attachment_ids = (array) $attachment_id;
        $placeholders = implode(", ", array_fill(0, count($attachment_ids), "(%d, 'pending')"));
        $query = "INSERT INTO {$this->table_name} (attachment_id, status) VALUES $placeholders";
    
        return $wpdb->query($wpdb->prepare($query, ...$attachment_ids));
    }
    
    public function update_status($attachment_id, $status, $error_code = null) {
        global $wpdb;

        $data = array(
            'status' => $status
        );
        $format = array('%s');

        if ($error_code !== null) {
            $data['error_code'] = $error_code;
            $format[] = '%s';
        }

        return $wpdb->update(
            $this->table_name,
            $data,
            array('attachment_id' => $attachment_id),
            $format,
            array('%d')
        );
    }

    public function get_status($attachment_id) {
        global $wpdb;

        return $wpdb->get_var($wpdb->prepare(
            "SELECT status FROM $this->table_name WHERE attachment_id = %d",
            $attachment_id
        ));
    }

    public function get_pending_images() {
        global $wpdb;

        return $wpdb->get_results(
            "SELECT attachment_id FROM $this->table_name WHERE status = 'pending'",
            ARRAY_A
        );
    }

    public function has_pending_images(){
        global $wpdb;

        return $wpdb->get_var("SELECT COUNT(*) FROM $this->table_name WHERE status = 'pending'") > 0;
    }

    public function get_next_image() {
        global $wpdb;
        return $wpdb->get_var("SELECT attachment_id FROM $this->table_name WHERE status = 'pending' LIMIT 1");
    }

    public function get_stats() {
        global $wpdb;

        $stats = $wpdb->get_row(
            "SELECT 
                COUNT(*) as total_images,
                SUM(CASE WHEN status = 'completed' THEN 1 ELSE 0 END) as total_completed,
                SUM(CASE WHEN status = 'failed' THEN 1 ELSE 0 END) as total_failed,
                SUM(CASE WHEN status = 'pending' THEN 1 ELSE 0 END) as total_pending
            FROM $this->table_name",
            ARRAY_A
        );

        $failed_images = $this->get_failed_images();
        $failed_images_details = array();
        foreach ($failed_images as $attachment_id => $error_code) {
            $error_message = Regenerate_Thumbnails::get_error_message($error_code);
            $attachment = get_post($attachment_id);
            $stats['failed_images_details'][$attachment_id] = array(
                'filename' => basename(get_attached_file($attachment_id)),
                'message' => "Skipped ({$attachment->post_title}), {$error_message}",
                'editLink' => get_edit_post_link($attachment_id, 'raw')
            );
        }
        $stats['total_processed'] = $stats['total_completed'] + $stats['total_failed'];

        $percentage = ($stats['total_images'] > 0) ? round(($stats['total_processed'] / $stats['total_images']) * 100) : 0;

        $stats =  wp_parse_args($stats, array(
            'total_images' => 0,
            'total_processed' => 0,
            'total_failed' => 0,
            'total_pending' => 0,
            'percentage' => $percentage,
            'failed_images_details' => $failed_images_details,
        ));

        $stats['progress_text'] = sprintf(
            /* translators: 1: processed images, 2: total images, 3: percentage, 4: failed images, 5: pending images */
            __( 'Processed %1$d out of %2$d images (%3$d%%). %4$d failed. %5$d pending.', 'rethumbify' ),
            $stats['total_processed']   ,
            $stats['total_images'],
            $stats['percentage'],
            $stats['total_failed'],
            $stats['total_pending']
        );

        return $stats;
    }

    public function clear() {
        global $wpdb;

        return $wpdb->query("TRUNCATE TABLE $this->table_name");
    }
}