<?php
/*
Plugin Name: ReThumbify
Description: Effortlessly regenerates your WordPress thumbnails—optimized for large image libraries.
Version: 1.0
Author: Nabil Lemsieh
Plugin URI: https://wordpress.org/plugins/rethumbify
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: rethumbify
Domain Path: /languages
*/

// If this file is called directly, abort.

if ( ! defined( 'WPINC' ) ) {
	die;
}

if (!(defined('WP_CLI') && WP_CLI) && function_exists('is_plugin_active') && function_exists('deactivate_plugins')):
    if (defined('RETHUMBIFY_PRO') && is_plugin_active('rethumbify-pro/rethumbify.php')) {
        deactivate_plugins('rethumbify-pro/rethumbify.php');
        wp_redirect($_SERVER['REQUEST_URI']);
        exit;
    }
    endif;

define('RETHUMBIFY_VERSION', '0.10.0');
define('RETHUMBIFY_DIR', plugin_dir_path(__FILE__));
define('RETHUMBIFY_URL', plugin_dir_url(__FILE__));
define('RETHUMBIFY_BASENAME', plugin_basename(__FILE__));

require_once plugin_dir_path(__FILE__) . 'includes/class-logger.php';
require_once plugin_dir_path(__FILE__) . 'includes/class-media-library.php';
require_once plugin_dir_path(__FILE__) . 'includes/class-status.php';
require_once plugin_dir_path(__FILE__) . 'includes/class-regenerate-thumbnails.php';
require_once plugin_dir_path(__FILE__) . 'includes/class-bulk-process.php';

function rethumbify_is_pro(){
    return defined('RETHUMBIFY_PRO') && RETHUMBIFY_PRO;
}

if( ! class_exists('Rethumbify') ){
class Rethumbify {
    private $bulkprocess;

    public function __construct() {

        $this->bulkprocess = new Rethumbify\Bulk_Process();
        
        add_action('plugins_loaded', array($this, 'load_textdomain'));
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        add_action('wp_ajax_rethumbify_start_regeneration', array($this, 'ajax_start_regeneration'));
        add_action('wp_ajax_rethumbify_get_progress', array($this, 'ajax_get_progress'));
        add_action('admin_init', array($this, 'register_settings'));
        add_action('wp_ajax_rethumbify_save_settings', array($this, 'ajax_save_settings'));
        add_action('wp_ajax_rethumbify_pause_regeneration', array($this, 'ajax_pause_regeneration'));
        add_action('wp_ajax_rethumbify_resume_regeneration', array($this, 'ajax_resume_regeneration'));
        add_filter('media_row_actions', array($this, 'add_regenerate_link'), 10, 2);
        add_filter('attachment_fields_to_edit', array($this, 'add_regenerate_button'), 10, 2);
        add_action('admin_action_rethumbify_regenerate_single', array($this, 'handle_regenerate_single_action'));
        add_action('admin_notices', array($this, 'display_admin_notices'));
        add_action('add_meta_boxes_attachment', array($this, 'add_rethumbify_metabox'));
        add_action('wp_ajax_rethumbify_regenerate_thumbnails', array($this, 'ajax_regenerate_thumbnails'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_media_scripts'));
        add_action('wp_ajax_rethumbify_retry_failed_image', array($this, 'ajax_retry_failed_image'));
        add_action('wp_ajax_rethumbify_process_next', array($this, 'ajax_process_next'));
        add_filter('plugin_action_links', array($this, 'add_plugin_page_links'), 10, 2);
        register_deactivation_hook(__FILE__, array($this, 'plugin_deactivation'));
        add_action('wp_ajax_rethumbify_get_subsizes', array($this, 'ajax_get_subsizes'));
        add_action('wp_ajax_rethumbify_get_logs', array($this, 'ajax_get_logs'));
        add_action('rethumbify_cleanup_logs', array(Rethumbify\Logger::get_instance(), 'cleanup_old_logs'));
        register_deactivation_hook(__FILE__, array($this, 'rethumbify_cleanup_cron'));
    }

    public function ajax_process_next(){
        check_ajax_referer('rethumbify_process_next', 'nonce');
        $this->bulkprocess->process_next();
        wp_send_json_success(array(
            'is_completed' => $this->bulkprocess->status->is_completed()
        ));
    }

    public function rethumbify_cleanup_cron() {
        wp_clear_scheduled_hook('rethumbify_cleanup_logs');
    }

    public function plugin_deactivation() {

        delete_option( 'rethumbify_completion_email_sent' );

        $status = new Rethumbify\Status();
        $status->clear();

        // Clear caches
        wp_cache_delete( 'rethumbify_total_images' );
        wp_cache_delete( 'rethumbify_image_ids' );

        // Flush object cache
        if(class_exists('ActionScheduler') && function_exists('as_unschedule_all_actions')){
            as_unschedule_all_actions('rethumbify_process_single_image');
        }

        wp_cache_flush();

    }

    public function ajax_retry_failed_image() {
        check_ajax_referer('rethumbify_retry_failed_nonce', 'nonce');
        
        $attachment_id = isset($_POST['attachment_id']) ? intval($_POST['attachment_id']) : 0;
        if (!$attachment_id) {
            wp_send_json_error(array('message' => __('Invalid attachment ID.', 'rethumbify')));
        }
    
        $failed_images = $this->bulkprocess->status->get_failed_images();

        if (!isset($failed_images[$attachment_id])) {
            wp_send_json_error(array('message' => __('This image is not in the failed images list.', 'rethumbify')));
        }
    
        // Attempt to regenerate the image
        $rt =  new Rethumbify\Regenerate_Thumbnails();
        $result = $rt->regenerate_thumbnails($attachment_id);
    
        if ($result === true) {
            $message = __('Thumbnails regenerated successfully.', 'rethumbify');
            wp_send_json_success(['message' => $message]);

        } else {
            // Regeneration failed, keep in failed list
            $error_message = is_wp_error($result) ? $result->get_error_message() : __('Unknown error', 'rethumbify');
            /* translators: %s: Error message */
            $message = sprintf(__('Failed to regenerate image. Error: %s', 'rethumbify'), $error_message);
            wp_send_json_error(['message' => $message]);
        }
    
    }

    public function load_textdomain() {
        load_plugin_textdomain('rethumbify', false, dirname(plugin_basename(__FILE__)) . '/languages');
    }

    public function add_admin_menu() {
        add_management_page('ReThumbify', 'ReThumbify', 'manage_options', 'rethumbify', array($this, 'admin_page'));
    }

    public function enqueue_media_scripts($hook) {
        if ($hook !== 'post' && $hook !== 'upload.php' && $hook !== 'post.php') {
            return;
        }

        $version = defined('WP_DEBUG') && WP_DEBUG ? time() : RETHUMBIFY_VERSION;

        wp_enqueue_style('dashicons');
        wp_enqueue_style('rethumbify-modal-css', plugins_url('css/rethumbify-modal.css', __FILE__), array(), $version);
        wp_enqueue_script('rethumbify-media-js', plugins_url('js/rethumbify-media.js', __FILE__), array('jquery'), $version, true);
        wp_localize_script('rethumbify-media-js', 'rethumbifyMediaData', array(
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'regenerateNonce' => wp_create_nonce('rethumbify_regenerate_nonce'),
            'subsizesNonce' => wp_create_nonce('rethumbify_subsizes_nonce'),
        ));
    }

    public function enqueue_admin_scripts($hook) {
        if ($hook !== 'tools_page_rethumbify') {
            return;
        }
        $version = defined('WP_DEBUG') && WP_DEBUG ? time() : RETHUMBIFY_VERSION;

        wp_enqueue_style('rethumbify-admin-css', plugins_url('css/rethumbify-admin.css', __FILE__), array(), $version);
        wp_enqueue_script('rethumbify-admin-js', plugins_url('js/rethumbify-admin.js', __FILE__), array('jquery'), $version, true);

        wp_localize_script('rethumbify-admin-js', 'rethumbifyData', array(
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'settingsNonce' => wp_create_nonce('rethumbify_settings_nonce'),
            'progressNonce' => wp_create_nonce('rethumbify_get_progress'),
            'startNonce' => wp_create_nonce('rethumbify_start_regeneration'),
            'pauseNonce' => wp_create_nonce('rethumbify_pause_regeneration'),
            'singleImageNonce' => wp_create_nonce('rethumbify_process_single_image'),
            'resumeNonce' => wp_create_nonce('rethumbify_resume_regeneration'),
            'retryAllNonce' => wp_create_nonce('rethumbify_retry_all_failed'),
            'retryFailedNonce' => wp_create_nonce('rethumbify_retry_failed_nonce'),
            'isRunning' => method_exists($this->bulkprocess, 'is_inprogress') ? $this->bulkprocess->is_inprogress() : false,
            'isPaused' => $this->bulkprocess->is_paused(),
            'stats' => $this->bulkprocess->status->get_stats(),
            'processNextNonce' => wp_create_nonce('rethumbify_process_next')
        ));
    }

    public function admin_page() {
        $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'dashboard';
        ?>
        <div class="wrap rethumbify">
        <h1 class="rethumbify-title">

                           ReThumbify <span class="rethumbify-version">v<?php echo RETHUMBIFY_VERSION; ?>-beta</span>
                   </h1>
        
        <h2 class="nav-tab-wrapper">
            <a href="?page=rethumbify&tab=dashboard" class="nav-tab <?php echo $active_tab === 'dashboard' ? 'nav-tab-active' : ''; ?>">Dashboard</a>
                        <a href="?page=rethumbify&tab=help" class="nav-tab <?php echo $active_tab === 'help' ? 'nav-tab-active' : ''; ?>">Help</a>
        </h2>
        
        <div class="rethumbify-container">
                <?php

                switch($active_tab){
                    case 'dashboard':
                        $this->render_bulk_regenerate_section();
                        $this->render_settings_section();
                        break;
                    case 'help':
                        $this->render_help_section();
                        break;
                }
                $this->render_footer();

                ?>
            </div>

           
        </div>
        <?php
    }

    private function render_footer() {
        ?>
         <div class="rethumbify-footer">
         <div class="rethumbify-footer-left">
         <div class="rethumbify-review">
         <?php 
         
$pro_link = '<a href="https://rethumbify.com/" target="_blank">ReThumbify Pro</a>';
$learn_more_link = '<a href="https://rethumbify.com/" target="_blank">' . esc_html__('Learn more 🡥', 'rethumbify') . '</a>';
         /* translators: 1: Plugin Website URL 2: Plugin Website URL */
    echo sprintf(
        esc_html__('Unlock more features with %1$s such as background regeneration, email notifications, selective regeneration, and more. %2$s', 'rethumbify'),
        $pro_link,
        $learn_more_link
    );
        ?>
           </div>
           </div>
        </div>

        <div class="rethumbify-footer">
            <div class="rethumbify-footer-left">
                    <div class="rethumbify-review">
                        <?php 
                        /* translators: %s: Plugin review URL */
                        printf(
                            wp_kses_post(__('Love ReThumbify? Please consider <a href="%s" target="_blank">leaving a 5-star review</a> to help us spread the word!', 'rethumbify')),
                            'https://wordpress.org/support/plugin/rethumbify/reviews/#new-post'
                        );
                        ?>
                    </div>
            </div>
        </div>
        <?php
    }
    private function render_help_section() {
        ?>
        <div class="rethumbify-section rethumbify-help-section">
            <div class="rethumbify-help-content">
            <div class="rethumbify-help-block">
                    <h2><?php esc_html_e('Need Help?', 'rethumbify'); ?></h2>
                    <p>
                        <?php esc_html_e('If you have any questions or need assistance, we\'re here to help!', 'rethumbify'); ?>
                    </p>
                    <p>
                        <a href="https://rethumbify.com/docs" target="_blank" class="button button-secondary">
                            <span class="dashicons dashicons-book"></span> <?php esc_html_e('Browse our documentation', 'rethumbify'); ?>
                        </a>
                        <a href="https://rethumbify.com/support.html" target="_blank" class="button button-secondary">
                            <span class="dashicons dashicons-sos"></span> <?php esc_html_e('Get Support', 'rethumbify'); ?>
                        </a>
                    </p>
                </div>
            </div>
            </div>
            <div class="rethumbify-section rethumbify-help-section">
            <div class="rethumbify-help-content">
                <div class="rethumbify-help-block">
                    <h2><?php esc_html_e('Debug Information', 'rethumbify'); ?></h2>
                    <p><?php esc_html_e('If you\'re experiencing issues, you can download the system information and logs below to share with support:', 'rethumbify'); ?></p>
                    
                    <h3><?php esc_html_e('System Information', 'rethumbify'); ?></h3>
                    <textarea id="rethumbify-debug-info" class="rethumbify-debug-info" style="width:100%;" readonly>
<?php
$debug_info = array(
    'WordPress Version' => get_bloginfo('version'),
    'PHP Version' => phpversion(),
    'Plugin Version' => RETHUMBIFY_VERSION,
    'Active Theme' => wp_get_theme()->get('Name'),
    'Memory Limit' => ini_get('memory_limit'),
    'Max Upload Size' => ini_get('upload_max_filesize'),
    'Max Post Size' => ini_get('post_max_size'),
    'Max Execution Time' => ini_get('max_execution_time'),
    'Image Sizes' => implode(', ', get_intermediate_image_sizes()),
    'Plugin Settings' => get_option('rethumbify_settings', array())
);
echo esc_textarea(wp_json_encode($debug_info, JSON_PRETTY_PRINT));
?></textarea>
                    <div class="rethumbify-button-group">
                        <button type="button" id="rethumbify-copy-debug" class="button button-secondary">
                            <span class="dashicons dashicons-clipboard"></span> <?php esc_html_e('Copy System Info', 'rethumbify'); ?>
                        </button>
                        <button type="button" id="rethumbify-download-debug" class="button button-secondary">
                            <span class="dashicons dashicons-download"></span> <?php esc_html_e('Download System Info', 'rethumbify'); ?>
                        </button>
                    </div>

                    <h3><?php esc_html_e('Regeneration Logs', 'rethumbify'); ?></h3>
                    <div class="rethumbify-button-group">
                        <button type="button" id="rethumbify-download-logs" class="button button-secondary">
                            <span class="dashicons dashicons-download"></span> <?php esc_html_e('Download Logs', 'rethumbify'); ?>
                        </button>
                    </div>
                </div>

            </div>
        </div>

        <script>
        jQuery(document).ready(function($) {
            function copyToClipboard(elementId, buttonId) {
                var content = $(elementId);
                content.select();
                document.execCommand('copy');
                
                var $button = $(buttonId);
                var originalText = $button.html();
                $button.html('<span class="dashicons dashicons-yes"></span> <?php esc_html_e("Copied!", "rethumbify"); ?>');
                
                setTimeout(function() {
                    $button.html(originalText);
                }, 2000);
            }

            function downloadContent(content, filename) {
                var blob = new Blob([content], { type: 'text/plain' });
                var url = window.URL.createObjectURL(blob);
                var a = document.createElement('a');
                a.href = url;
                a.download = filename;
                document.body.appendChild(a);
                a.click();
                window.URL.revokeObjectURL(url);
                document.body.removeChild(a);
            }

            $('#rethumbify-copy-debug').on('click', function() {
                copyToClipboard('#rethumbify-debug-info', '#rethumbify-copy-debug');
            });

            $('#rethumbify-download-debug').on('click', function() {
                var content = $('#rethumbify-debug-info').val();
                var date = new Date().toISOString().slice(0,10);
                downloadContent(content, 'rethumbify-debug-' + date + '.txt');
            });

            $('#rethumbify-download-logs').on('click', function() {
                $.ajax({
                    url: ajaxurl,
                    type: 'POST',
                    data: {
                        action: 'rethumbify_get_logs',
                        nonce: '<?php echo wp_create_nonce('rethumbify_get_logs'); ?>'
                    },
                    success: function(response) {
                        if (response.success) {
                            var date = new Date().toISOString().slice(0,10);
                            downloadContent(response.data, 'rethumbify-logs-' + date + '.txt');
                        }
                    }
                });
            });
        });
        </script>
        <?php
    }

  

    private function render_bulk_regenerate_section() {
        $is_inprogress = $this->bulkprocess->is_inprogress();
        $is_inactive = $this->bulkprocess->is_inactive();
        $is_paused = $this->bulkprocess->is_paused();
        $paused_info = __('Thumbnail regeneration was interrupted. Click the "Resume Regeneration" button below to continue, or "Restart Regeneration" to begin again.', 'rethumbify');
        $inactive_info = __('Before starting, you can adjust the regeneration settings below. When ready, click the button below to begin regenerating thumbnails for your media library images.', 'rethumbify');

        

        if( rethumbify_is_pro() ){
            $inprogress_info = __('Feel free to close this page while ReThumbify Pro regenerates thumbnails in the background.', 'rethumbify');
        }
        else{
            /* translators: %s: Upgrade link */

            $upgrade_link = '<a href="https://rethumbify.com/" target="_blank" class="rethumbify-upgrade-link">' . esc_html__('Upgrade to ReThumbify Pro', 'rethumbify') . '</a>'; 
            $inprogress_info = sprintf(
                __('Hate waiting? %s and let thumbnails regenerate seamlessly in the background. You can also be notified when they\'re done!', 'rethumbify'),
                $upgrade_link
            );
        }

        $options = get_option('rethumbify_settings');
        $admin_email = !empty($options['notification_email']) ? $options['notification_email'] : get_option('admin_email');
        
        /* translators: %s: Admin email */
        $notif_email_info = sprintf(
                            __('We will email you at %s once the process is complete.', 'rethumbify'),
                                '<strong>' . esc_html($admin_email) . '</strong>');
        ?>
        <div class="rethumbify-section">
            <h2><?php esc_html_e('Bulk Regenerate', 'rethumbify'); ?></h2>
            
            <p class="rethumbify-bulk-regenerate-info" 
            data-paused-info="<?php echo esc_attr($paused_info) ?>"
            data-notif-email-info = "<?php echo esc_attr($notif_email_info) ?>"
            data-inactive-info="<?php echo esc_attr($inactive_info); ?>"
            data-inprogress-info="<?php echo esc_attr($inprogress_info); ?>">
            <?php
            if( $is_paused ){
                echo esc_html($paused_info);
            }elseif($is_inprogress){
                echo esc_html($inprogress_info);
            }
            elseif($is_inactive){
                echo esc_html($inactive_info);
            }
            ?>
            </p>
            <button type="button" id="rethumbify-resume-regeneration" class="button button-secondary" <?php echo $is_paused ? '' : 'style="display:none;"'; ?>>
                <?php esc_html_e('Resume Regeneration', 'rethumbify'); ?>
            </button>
            <button type="button" id="rethumbify-start-regeneration" class="button button-primary" <?php echo $is_inprogress ? 'style="display:none;"' : ''; ?>>
                <?php 
                if ($is_paused) {
                    esc_html_e('Restart Regeneration', 'rethumbify');
                } else {
                    esc_html_e('Start Regeneration', 'rethumbify');
                }
                ?>
            </button>
            
            <div id="rethumbify-progress-container" <?php echo $is_inprogress ? '' : 'style="display:none;"'; ?>>
                <div class="spinner rethumbify-spinner" style="float:none; margin:0 10px 0 0; <?php echo $is_inprogress ? '' : 'display:none;'; ?>"></div>
                <div id="rethumbify-progress-bar">
                    <div id="rethumbify-progress"></div>
                </div>
                
                <button type="button" id="rethumbify-pause-regeneration" class="button button-secondary button-small" <?php echo $is_inprogress ? '' : 'style="display:none;"'; ?>>
                    <?php esc_html_e('Pause', 'rethumbify'); ?>
                </button>
            </div>
            <p id="rethumbify-progress-text"></p>
        </div>
        <?php
    }

    private function render_settings_section() {
        ?>
        <div class="rethumbify-section">
            <h2><?php esc_html_e('Settings', 'rethumbify'); ?></h2>
            <form id="rethumbify-settings-form" method="post" action="options.php">
                <?php
                settings_fields('rethumbify_settings');
                do_settings_sections('rethumbify_settings');
                ?>
                <div class="rethumbify-submit-button-wrapper">
                    <input type="submit" name="submit" id="submit" class="button button-primary" value="<?php esc_html_e('Save Changes', 'rethumbify'); ?>">
                    <span class="spinner rethumbify-spinner"></span>
                    <span id="rethumbify-settings-message" class="rethumbify-settings-message"></span>
                </div>
            </form>
        </div>
        <?php
    }

    public function ajax_save_settings() {
        check_ajax_referer('rethumbify_settings_nonce', 'nonce');
    
        if (!current_user_can('manage_options')) {
            wp_send_json_error(__('You do not have permission to perform this action.', 'rethumbify'));
        }
    
        $settings = isset($_POST['settings']) ? $_POST['settings'] : array();
        $sanitized_settings = $this->sanitize_settings($settings);
        update_option('rethumbify_settings', $sanitized_settings);
        wp_cache_delete('rethumbify_total_images');
        wp_cache_delete('rethumbify_image_ids');
        wp_cache_flush();
        
        wp_send_json_success(__('Settings saved successfully.', 'rethumbify'));
    }

    public function register_settings() {
        register_setting(
        'rethumbify_settings',
        'rethumbify_settings',
        array( $this, 'sanitize_settings' )
    
    );

        add_settings_section(
            'rethumbify_general_settings',
            '',
            array($this, 'general_settings_section_callback'),
            'rethumbify_settings'
        );
    
        // add_settings_field(
        //     'rethumbify_regenerate_sizes',
        //     __('Sizes to Regenerate', 'rethumbify'),
        //     array( $this, 'regenerate_sizes_callback' ),
        //     'rethumbify_settings',
        //     'rethumbify_general_settings'
        // );

    add_settings_field(
        'rethumbify_regenerate_featured',
        __('Regenerate Featured Images', 'rethumbify'),
        array( $this, 'regenerate_featured_callback' ),
        'rethumbify_settings',
        'rethumbify_general_settings'
    );

    add_settings_field(
        'rethumbify_selective_regeneration',
        __('Selective Regeneration', 'rethumbify') . (defined('RETHUMBIFY_PRO') && RETHUMBIFY_PRO ? '' : ' <span class="rethumbify-pro-badge">PRO</span>'),
        array( $this, 'selective_regeneration_callback' ),
        'rethumbify_settings',
        'rethumbify_general_settings'
    );

    add_settings_field(
        'rethumbify_notify_on_completion',
        __('Notify on Completion', 'rethumbify') . (defined('RETHUMBIFY_PRO') && RETHUMBIFY_PRO ? '' : ' <span class="rethumbify-pro-badge">PRO</span>'),
        array($this, 'notify_on_completion_callback'),
        'rethumbify_settings',
        'rethumbify_general_settings'
    );

    }

    public function general_settings_section_callback() {
        echo '';
    }

    public function sanitize_settings($input) {
        $sanitized_input = array();
    
        if (isset($input['regenerate_mode'])) {
            $sanitized_input['regenerate_mode'] = sanitize_text_field($input['regenerate_mode']);
        }
    
        if (isset($input['custom_sizes']) && is_array($input['custom_sizes'])) {
            $sanitized_input['custom_sizes'] = array_map('sanitize_text_field', $input['custom_sizes']);
        } else {
            $sanitized_input['custom_sizes'] = array();
        }
    
        if (isset($input['regenerate_featured'])) {
            $sanitized_input['regenerate_featured'] = (bool) $input['regenerate_featured'];
        }
    
        if (isset($input['notify_on_completion'])) {
            $sanitized_input['notify_on_completion'] = (bool) $input['notify_on_completion'];
        }
    
        if (isset($input['selective_rules']) && is_array($input['selective_rules'])) {
            $sanitized_input['selective_rules'] = array();
            foreach ($input['selective_rules'] as $rule) {
                $sanitized_rule = array(
                    'condition' => sanitize_text_field($rule['condition']),
                );
                if ($rule['condition'] === 'upload_date' && !empty($rule['upload_date'])) {
                    $sanitized_rule['upload_date'] = sanitize_text_field($rule['upload_date']);
                } elseif ($rule['condition'] === 'post_type' && !empty($rule['post_type'])) {
                    $sanitized_rule['post_type'] = sanitize_text_field($rule['post_type']);
                } else {
                    // Skip this rule if no value is entered
                    continue;
                }
                $sanitized_input['selective_rules'][] = $sanitized_rule;
            }
        }
     
    
    
        // Add email sanitization
        if (isset($input['notification_email'])) {
            $email = sanitize_email($input['notification_email']);
            if (!empty($email)) {
                $sanitized_input['notification_email'] = $email;
            }
        }

        return $sanitized_input;
    }

    public function regenerate_sizes_callback() {
        $options = get_option( 'rethumbify_settings' );
        $regenerate_mode = isset( $options['regenerate_mode'] ) ? $options['regenerate_mode'] : 'all';
        $custom_sizes = isset( $options['custom_sizes'] ) ? $options['custom_sizes'] : array();
        
        $registered_sizes = get_intermediate_image_sizes();
        ?>
        <p>
            <label>
                <input type="radio" name="rethumbify_settings[regenerate_mode]" value="all" <?php checked( 'all', $regenerate_mode ); ?>>
                Regenerate all sizes
            </label>
        </p>
        <p>
            <label>
                <input type="radio" name="rethumbify_settings[regenerate_mode]" value="custom" <?php checked( 'custom', $regenerate_mode ); ?>>
                Regenerate selected sizes
            </label>
        </p>
        <div id="rethumbify-custom-sizes" style="margin-left: 20px; <?php echo $regenerate_mode === 'custom' ? '' : 'display: none;'; ?>">
            <?php foreach ( $registered_sizes as $size ) : ?>
                <label>
                    <input type="checkbox" name="rethumbify_settings[custom_sizes][]" value="<?php echo esc_attr( $size ); ?>" 
                        <?php checked( in_array( $size, $custom_sizes ) || $regenerate_mode !== 'custom' ); ?>>
                    <?php echo esc_html( $size ); ?>
                </label><br>
            <?php endforeach; ?>
        </div>
        <script>
        jQuery(document).ready(function($) {
            var $customSizes = $('#rethumbify-custom-sizes');
            var $customCheckboxes = $customSizes.find('input[type="checkbox"]');
            
            $('input[name="rethumbify_settings[regenerate_mode]"]').change(function() {
                if ($(this).val() === 'custom') {
                    $customSizes.show();
                    $customCheckboxes.prop('checked', true);
                } else {
                    $customSizes.hide();
                }
            });
    
            // If it's the first time selecting custom, check all boxes
            if ($('input[name="rethumbify_settings[regenerate_mode]"]:checked').val() !== 'custom') {
                $customCheckboxes.prop('checked', true);
            }
        });
        </script>
        <?php
    }
    
    public function regenerate_featured_callback() {
    $options = get_option( 'rethumbify_settings' );
    $regenerate_featured = isset( $options['regenerate_featured'] ) ? $options['regenerate_featured'] : false;
    ?>
    <label>
        <input type="checkbox" class="rethumbify-toggle" name="rethumbify_settings[regenerate_featured]" value="1" <?php checked( 1, $regenerate_featured ); ?>>
        Regenerate only featured images
    </label>
    <p class="description">This will regenerate only the main images attached to posts as featured images.</p>
    <?php
}

public function selective_regeneration_callback() {
    
    ?>
            <div class="rethumbify-add-rule">
            <button type="button" id="rethumbify-add-selective-rule" class="button <?php if(! rethumbify_is_pro()){echo 'rethumbify-pro-feature';} ?>" class="button" <?php if( ! rethumbify_is_pro() ){echo 'disabled';} ?>>Add Rule</button>
        </div>
       
    </div>
    <p class="description">
    Choose specific images to regenerate based on your needs, saving both time and server resources.
        </p>
            <?php
}

public function notify_on_completion_callback() {

    ?>
    <div>
        <input type="checkbox" class="rethumbify-toggle" id="rethumbify_notify_on_completion" name="rethumbify_settings[notify_on_completion]" value="1" 
        
        <?php 
        echo rethumbify_is_pro() ? '' : 'disabled' 
    ?>
        <?php echo rethumbify_is_pro() ? 'disabled' : '' ?>
        >
        <label for="rethumbify_notify_on_completion"><?php esc_html_e('Send an email notification when the process is complete', 'rethumbify'); ?></label>
    </div>
        <?php
}

    public function ajax_start_regeneration() {
        check_ajax_referer('rethumbify_start_regeneration', 'nonce');
        
            $total_images = $this->bulkprocess->start();
            
            if ($total_images === 0) {
                wp_send_json_error(array(
                    'message' => sprintf(__('No images found based on the current settings. Please check your selective regeneration rules in the plugin settings.', 'rethumbify')),
                ));
            } else {
                wp_send_json_success(array(
                    'message' => __('Regeneration process started successfully.', 'rethumbify'),
                    'stats' => $this->bulkprocess->status->get_stats()
                ));
            }
    }

    public function ajax_get_progress() {
        check_ajax_referer('rethumbify_get_progress', 'nonce');
        $stats = $this->bulkprocess->status->get_stats();
        $stats['is_completed'] = $this->bulkprocess->is_completed();
        wp_send_json_success( $stats );
    }

    public function ajax_pause_regeneration() {
        check_ajax_referer('rethumbify_pause_regeneration', 'nonce');
       
        if($this->bulkprocess->pause()){
            wp_send_json_success(array(
                'message' => __('Regeneration process has been interrupted. Click the "Resume Regeneration" button to continue.', 'rethumbify')
            ));
        }else{
            wp_send_json_error(array('message' => __('Failed to pause regeneration process.', 'rethumbify')));
        }

       
    }

    public function ajax_resume_regeneration() {
        check_ajax_referer('rethumbify_resume_regeneration', 'nonce');
        
        if ($this->bulkprocess->resume()) {
            wp_send_json_success(array(
                'message' => __('Regeneration process resumed successfully.', 'rethumbify'),
                'stats' => $this->bulkprocess->status->get_stats()
            ));
        } else {
            wp_send_json_error(array('message' => __('Failed to resume regeneration process.', 'rethumbify')));
        }
    }

    public function add_regenerate_link($actions, $post) {
        if ('image/' === substr($post->post_mime_type, 0, 6)) {
            $url = $this->get_regenerate_url($post->ID);
            $actions['rethumbify_regenerate'] = '<a href="' . esc_url($url) . '" title="' . esc_attr__('Regenerate Thumbnails with ReThumbify', 'rethumbify') . '">' . __('Regenerate Thumbnails', 'rethumbify') . '</a>';
            $actions['rethumbify_sizes'] = '<a href="#" class="rethumbify-show-subsizes" data-attachment-id="' . esc_attr($post->ID) . '">' . __('View Thumbnails', 'rethumbify') . '</a>';
        }
        return $actions;
    }

    public function add_regenerate_button($form_fields, $post) {
        // Check if we're in a modal view
        if(wp_doing_ajax()){
            if ('image/' === substr($post->post_mime_type, 0, 6)) {
                $form_fields['rethumbify_regenerate'] = array(
                    'label' => 'ReThumbify',
                    'input' => 'html',
                    'html' => '<button type="button" class="button-secondary rethumbify-regenerate-button" data-attachment-id="' . esc_attr($post->ID) . '">' . 
                        __('Regenerate Thumbnails', 'rethumbify') . 
                        '</button>' .
                        '<a href="#" class="rethumbify-show-subsizes" data-attachment-id="' . esc_attr($post->ID) . '" style="display: block; margin-top: 5px;">' . 
                        __('View Thumbnails', 'rethumbify') . 
                        '</a>'
                );
            }
        }
        return $form_fields;
    }

    private function get_regenerate_url($post_id) {
        return wp_nonce_url(
            admin_url('admin.php?action=rethumbify_regenerate_single&post=' . $post_id),
            'rethumbify_regenerate_' . $post_id
        );
    }

    public function handle_regenerate_single_action() {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.', 'rethumbify'));
        }

        $post_id = isset($_GET['post']) ? intval($_GET['post']) : 0;
        if (!$post_id) {
            wp_die(__('No image ID specified.', 'rethumbify'));
        }

        check_admin_referer('rethumbify_regenerate_' . $post_id);

        $rt = new Rethumbify\Regenerate_Thumbnails();
        $result = $rt->regenerate_thumbnails($post_id);
        $image_file_name = get_post_meta($post_id, '_wp_attached_file', true);
        $image_file_name = basename($image_file_name);

        if ($result === true) {
            /* translators: %s: Image file name */
            $message = sprintf(__('Thumbnails regenerated successfully for image: %s', 'rethumbify'), $image_file_name);
            $type = 'success';
        } else {
            $error_message = is_wp_error($result) ? $result->get_error_message() : __('Unknown error', 'rethumbify');
            /* translators: 1: Image file name 2: Error message */
            $message = sprintf(__('Failed to regenerate thumbnails for image: %1$s. Error: %2$s', 'rethumbify'), $image_file_name, $error_message);
            $type = 'error';
        }

        $redirect_url = wp_get_referer();
        if (!$redirect_url) {
            $redirect_url = admin_url('upload.php');
        }
        wp_redirect(add_query_arg(array('rethumbify_message' => urlencode($message), 'rethumbify_type' => $type), $redirect_url));
        exit;
    }

    public function display_admin_notices() {
        if (isset($_GET['rethumbify_message']) && isset($_GET['rethumbify_type'])) {
            $message = urldecode($_GET['rethumbify_message']);
            $type = $_GET['rethumbify_type'];
            ?>
            <div class="notice notice-<?php echo esc_attr($type); ?> is-dismissible">
                <p><?php echo esc_html($message); ?></p>
            </div>
            <?php
        }
    }

    public function add_rethumbify_metabox($post) {
        if (wp_attachment_is_image($post->ID)) {
            add_meta_box(
                'rethumbify_metabox',
                __('ReThumbify', 'rethumbify'),
                array($this, 'render_rethumbify_metabox'),
                'attachment',
                'side',
                'core'
            );
        }
    }

    public function render_rethumbify_metabox($post) {
        $url = $this->get_regenerate_url($post->ID);
        ?>
        <p><?php esc_html_e('Click the button below to regenerate thumbnails for this image.', 'rethumbify'); ?></p>
        <a href="<?php echo esc_url($url); ?>" class="button button-secondary">
            <?php esc_html_e('Regenerate Thumbnails', 'rethumbify'); ?>
        </a>
        <p class="rethumbify-subsizes">
            <a href="#" class="rethumbify-show-subsizes" data-attachment-id="<?php echo esc_attr($post->ID); ?>">
                <?php esc_html_e('View Thumbnails', 'rethumbify'); ?>
            </a>
        </p>
        <?php
    }

   

    public function ajax_regenerate_thumbnails() {
        check_ajax_referer('rethumbify_regenerate_nonce', 'nonce');
        $attachment_id = isset($_POST['attachment_id']) ? intval($_POST['attachment_id']) : 0;
        if (!$attachment_id) {
            wp_send_json_error(array('message' => __('No image ID specified.', 'rethumbify')));
        }

        $rt = new Rethumbify\Regenerate_Thumbnails();
        $result = $rt->regenerate_thumbnails($attachment_id);
        $image_file_name = get_post_meta($attachment_id, '_wp_attached_file', true);
        $image_file_name = basename($image_file_name);

        if ($result === true) {
            /* translators: %s: Image file name */
            wp_send_json_success(array(
                'message' => sprintf(__('Thumbnails regenerated successfully for image: %s', 'rethumbify'), $image_file_name)
            ));
        } else {
            $error_message = is_wp_error($result) ? $result->get_error_message() : __('Unknown error', 'rethumbify');
            /* translators: 1: Image file name 2: Error message */
            wp_send_json_error(array(
                'message' => sprintf(__('Failed to regenerate thumbnails for image: %1$s. Error: %2$s', 'rethumbify'), $image_file_name, $error_message)
            ));
        }
    }

    public function add_plugin_page_links($links, $file) {
        if (RETHUMBIFY_BASENAME === $file) {
            $dashboard_url = admin_url('tools.php?page=rethumbify');
            $docs_url = 'https://rethumbify.com/docs';
    
            $new_links = array(
                'dashboard' => sprintf('<a href="%s">%s</a>', esc_url($dashboard_url), __('Dashboard', 'rethumbify')),
                'docs' => sprintf('<a href="%s" target="_blank">%s</a>', esc_url($docs_url), __('Docs', 'rethumbify'))
            );
            
            $upgrade_url = 'https://rethumbify.com';
            $new_links['upgrade'] = sprintf('<a href="%s" target="_blank" style="color: #ff5722; font-weight: bold;">%s</a>', esc_url($upgrade_url), __('Upgrade to Pro', 'rethumbify'));
            $links = array_merge($new_links, $links);
        }
        return $links;
    }

    public function ajax_get_subsizes() {
        check_ajax_referer('rethumbify_subsizes_nonce', 'nonce');
        
        $attachment_id = isset($_POST['attachment_id']) ? intval($_POST['attachment_id']) : 0;
        if (!$attachment_id) {
            wp_send_json_error();
        }

        $metadata = wp_get_attachment_metadata($attachment_id);
        
        if (!empty($metadata['sizes'])) {
            foreach ($metadata['sizes'] as $size_name => $size_data) {
                $image_src = wp_get_attachment_image_src($attachment_id, $size_name);
                if ($image_src) {
                    $sizes[$size_name] = array(
                        'width' => $size_data['width'],
                        'height' => $size_data['height'],
                        'url' => $image_src[0]
                    );
                }
            }
        }

        wp_send_json_success($sizes);
    }

    public function ajax_get_logs() {
        check_ajax_referer('rethumbify_get_logs', 'nonce');

        if (!current_user_can('manage_options')) {
            wp_send_json_error(__('You do not have permission to perform this action.', 'rethumbify'));
        }

        $logger = \Rethumbify\Logger::get_instance();
        $logs = $logger->get_log_contents();

        if (empty($logs)) {
            $logs = __('No logs available.', 'rethumbify');
        }

        wp_send_json_success($logs);
    }
}

}

new Rethumbify();